package spinn3rreaderd;

import java.io.BufferedWriter;
import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * @author west1 (adapting code by klemen)
 */
public class Entry {
	public String m_URL = null;
	public String m_title = null;
	public String m_htmlTitle = null;
	public String m_pubDate = null;
	public String m_content = null;
	public String m_htmlContent = null;
	public ArrayList<StrPosLenTriple> m_URLs = null;
	public ArrayList<PosLenPair> m_quotes = null;

	private static final Pattern SENTENCE_PATTERN = Pattern.compile("(.*? [.?!]) .*");
	private static final int MAX_QUOTE_LENGTH = 300;
	private static final int MIN_QUOTE_LENGTH = 4;
	private static final char SEPARATOR = '\t';

	//
	public Entry() {
	}

	// sometime quotes can be very long, either intentionally, or because the quote detection method failed
	// because of ill-formated quotes (e.g., ''abc`` def ''geh`` yields the quote " def "); in these cases,
	// just use the first sentence.
	// also, we require quotes to be of a length of more than 3 chars.
	// if the input doesn't yield a valid quote (too short or no sentence boundary), throw
	// IllegalArgumentException
	private static String normalizeQuote(String quote) {
		if (quote.length() < MIN_QUOTE_LENGTH) {
			throw new IllegalArgumentException(quote);
		} else if (quote.length() > MAX_QUOTE_LENGTH) {
			Matcher firstSentenceMatcher = SENTENCE_PATTERN.matcher(quote);
			if (firstSentenceMatcher.matches()) {
				return firstSentenceMatcher.group(1);
			} else {
				throw new IllegalArgumentException(quote);
			}
		} else {
			return quote;
		}
	}

	public void write(BufferedWriter bw) throws Exception {
		// URL -- U
		bw.write("U:");
		bw.write(m_URL);
		bw.write(SEPARATOR);
		//
		// Date -- D
		bw.write("D:");
		bw.write(m_pubDate);
		bw.write(SEPARATOR);
		//
		// Title -- T
		bw.write("T:");
		bw.write(m_title);
		bw.write(SEPARATOR);
		//
		// HTML Title -- F
		bw.write("F:");
		bw.write(m_htmlTitle);
		bw.write(SEPARATOR);
		//
		// Content -- C
		if (m_content != null) {
			bw.write("C:");
			bw.write(m_content);
			bw.write(SEPARATOR);
		}
		//
		// HTML Content -- H
		if (m_htmlContent != null) {
			bw.write("H:");
			bw.write(m_htmlContent);
			bw.write(SEPARATOR);
		}
		//
		// URLs - L
		// NB: this pattern ignores hyperlinks on anchors of length zero, e.g.,
		// <a href="http://..."></a>
		if (m_URLs != null) {
			for (StrPosLenTriple l : m_URLs) {
				// If a URL appears in the title, its position and length are undefined, because we let pos
				// and len always refer to indices in the plain-text content; 'undefined' is encoded as -1.
				bw.write(String.format("L:%d:%d:%s", l.pos, l.len, l.str));
				bw.write(SEPARATOR);
			}
		}
		//
		// Quotes - Q
		if (m_quotes != null) {
			for (PosLenPair p : m_quotes) {
				try {
					bw.write(String.format("Q:%d:%d:%s", p.pos, p.len, normalizeQuote(m_content.substring(p.pos, p.pos + p.len))));
					bw.write(SEPARATOR);
				} catch (IllegalArgumentException e) {
					// this is thrown by normalizeQuote() in some abnormal cases (see documentation there)
				}
			}
		}
		//
		// Newline
		bw.write('\n');
	}
}
